/*
 * Copyright 2008 Chris Povirk
 * 
 * This file is part of General Manager for Tecmo Super Bowl.
 * 
 * General Manager for Tecmo Super Bowl is free software: you can redistribute
 * it and/or modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 * 
 * General Manager for Tecmo Super Bowl is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * General Manager for Tecmo Super Bowl. If not, see
 * <http://www.gnu.org/licenses/>.
 */

package org.twofoos.gmtsb.core;

import org.twofoos.gmtsb.util.iterators.AbstractMapper;
import org.twofoos.gmtsb.util.iterators.Mapper;

public enum Attribute
{
  RS("RS", "Running Speed"), //
  RP("RP", "Rushing Power"), //
  MS("MS", "Maximum Speed"), //
  HP("HP", "Hitting Power"),

  BC("BC", "Ball Control"), //
  Rec("Rec", "Receptions"),

  PS("PS", "Passing Speed"), //
  PC("PC", "Pass Control"), //
  PA("PA", "Accuracy of Passing"), /** Avoid PASS Rush */
  AR("AR", "Avoid Pass Block"),

  KA("KA", "Kicking Ability"), /** Avoid KICK Block */
  AB("AB", "Avoid Kick Block"),

  Int("Int", "Interceptions"), Qui("Qui", "Quickness");

  private final String abbreviation;
  private final String name;

  private Attribute(final String abbreviation, final String name)
  {
    // Not publically instantiable.
    this.abbreviation = abbreviation;
    this.name = name;
  }

  public String getAbbreviation()
  {
    return abbreviation;
  }

  public String getName()
  {
    return name;
  }

  @Override
  public String toString()
  {
    return getAbbreviation();
  }

  public static final Mapper<Attribute, Attribute> DISPLAY_ORDER_MAPPER =
      new DisplayOrderMapper();

  private static class DisplayOrderMapper extends
      AbstractMapper<Attribute, Attribute>
  {
    @Override
    public Attribute apply(Attribute input)
    {
      if(input == Attribute.RS)
      {
        return Attribute.RP;
      }

      if(input == Attribute.RP)
      {
        return Attribute.RS;
      }

      return input;
    }
  }
}
