/*
 * Copyright 2008 Chris Povirk
 * 
 * This file is part of General Manager for Tecmo Super Bowl.
 * 
 * General Manager for Tecmo Super Bowl is free software: you can redistribute
 * it and/or modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 * 
 * General Manager for Tecmo Super Bowl is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * General Manager for Tecmo Super Bowl. If not, see
 * <http://www.gnu.org/licenses/>.
 */

package org.twofoos.gmtsb.userevent.intrinsic;

import org.twofoos.gmtsb.core.TSBTransferableData;
import org.twofoos.gmtsb.userevent.UserEvent;

public abstract class IntrinsicEvent<D extends TSBTransferableData<D>, A>
    implements UserEvent
{
  private final D subject;
  private final A from;
  private final A to;

  protected IntrinsicEvent(D subject, A from, A to)
  {
    this.subject = subject;
    this.from = from;
    this.to = to;
  }

  public D getSubject()
  {
    return subject;
  }

  protected A getFrom()
  {
    return from;
  }

  protected A getTo()
  {
    return to;
  }

  protected abstract void changeSubjectAttribute(A value);

  // TODO remove final on these

  public final boolean doEvent()
  {
    changeSubjectAttribute(to);
    return !from.equals(to);
  }

  public final void undoEvent()
  {
    changeSubjectAttribute(from);
  }
}
