/*
 * Copyright 2008 Chris Povirk
 * 
 * This file is part of General Manager for Tecmo Super Bowl.
 * 
 * General Manager for Tecmo Super Bowl is free software: you can redistribute
 * it and/or modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 * 
 * General Manager for Tecmo Super Bowl is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * General Manager for Tecmo Super Bowl. If not, see
 * <http://www.gnu.org/licenses/>.
 */

package org.twofoos.gmtsb.userevent.intrinsic;

import org.twofoos.gmtsb.core.TSBTransferableData;
import org.twofoos.gmtsb.userevent.UserEvent;

public abstract class MergeableIntrinsicEvent<T extends MergeableIntrinsicEvent<T, D, A>, D extends TSBTransferableData<D>, A>
    extends IntrinsicEvent<D, A>
{
  private final Object source;

  protected MergeableIntrinsicEvent(D subject, A from, A to, Object source)
  {
    super(subject, from, to);
    this.source = source;
  }

  public Object getSource()
  {
    return source;
  }

  protected abstract UserEvent newEventForMerge(A from, A to);

  @SuppressWarnings("unchecked")
  public final UserEvent attemptToMergeWithPrevious(UserEvent previousEvent)
  {
    if(getClass().isInstance(previousEvent))
    {
      MergeableIntrinsicEvent other = (MergeableIntrinsicEvent) previousEvent;

      if(other.getSubject() == getSubject() && other.getSource() == getSource())
      {
        return newEventForMerge(((T) previousEvent).getFrom(), getTo());
      }
    }

    return null;
  }
}
