/*
 * Copyright 2008 Chris Povirk
 * 
 * This file is part of General Manager for Tecmo Super Bowl.
 * 
 * General Manager for Tecmo Super Bowl is free software: you can redistribute
 * it and/or modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 * 
 * General Manager for Tecmo Super Bowl is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * General Manager for Tecmo Super Bowl. If not, see
 * <http://www.gnu.org/licenses/>.
 */

package org.twofoos.gmtsb.userevent.intrinsic;

import org.twofoos.gmtsb.core.Notable;
import org.twofoos.gmtsb.core.TSBTransferableData;
import org.twofoos.gmtsb.userevent.UserEvent;

public class NotesChangeEvent<D extends TSBTransferableData<D> & Notable>
    extends MergeableIntrinsicEvent<NotesChangeEvent<D>, D, String>
{
  private NotesChangeEvent(D subject, String from, String to, Object source)
  {
    super(subject, from, to, source);
  }

  @Override
  protected UserEvent newEventForMerge(String from, String to)
  {
    return new NotesChangeEvent<D>(getSubject(), from, to, getSource());
  }

  @Override
  protected void changeSubjectAttribute(String value)
  {
    getSubject().setNotes(value);
  }

  public String getDescription()
  {
    return "Changed notes for " + getSubject();
  }

  /*
   * Not using a public constructor to be consistent with PlayerMoveEvent and to
   * make explicit type parameters unnecessary.
   */
  public static <D extends TSBTransferableData<D> & Notable> NotesChangeEvent<D> createNotesChangeEvent(
      D subject, String from, String to, Object source)
  {
    return new NotesChangeEvent<D>(subject, from, to, source);
  }
}
