/*
 * Copyright 2008 Chris Povirk
 * 
 * This file is part of General Manager for Tecmo Super Bowl.
 * 
 * General Manager for Tecmo Super Bowl is free software: you can redistribute
 * it and/or modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 * 
 * General Manager for Tecmo Super Bowl is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * General Manager for Tecmo Super Bowl. If not, see
 * <http://www.gnu.org/licenses/>.
 */

package org.twofoos.gmtsb.util.iterators;

import java.util.Iterator;

public class ByteArrayIterable implements Iterable<Byte>
{
  private final byte[] array;

  public ByteArrayIterable(byte[] array)
  {
    this.array = array;
  }

  private static class ByteArrayIterator implements Iterator<Byte>
  {
    private final byte[] array;
    private int nextIndex = 0;

    public ByteArrayIterator(byte[] array)
    {
      this.array = array;
    }

    public boolean hasNext()
    {
      return nextIndex < array.length;
    }

    public Byte next()
    {
      return array[nextIndex++];
    }

    public void remove()
    {
      throw new UnsupportedOperationException("remove() not supported");
    }
  }

  public Iterator<Byte> iterator()
  {
    return new ByteArrayIterator(array);
  }
}
